%% Housekeeping %%%%%
clear all; close all;
%%%%%%%%%%%%%%%%%%%%%%
warning off all
%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load common variables
%%%%%%%%%%%%%%%%%%%%%%%%%
load p.txt
load S.txt
load e.txt 
load w_vec.txt 
load N_mu.txt 
load grid_num.txt  
load gam.txt  
load th.txt 
load del.txt  
load k.txt  
load alp.txt 
%% Utilities and policies
load U1.txt
load U2.txt
load ww.txt
load wp.txt

%% Shrink the grid
for s1=1:S
    U(s1,2:grid_num+2,1)=interp1(w_vec,U1(s1,:),ww(2:grid_num+2),'spline');
    U(s1,1:grid_num+1,2)=interp1(w_vec,U2(s1,:),ww(1:grid_num+1),'spline');
end
clear U1 U2;
U1=U(:,:,1); U2=U(:,:,2);
clear U;

%%Relabeling
w_inv=wp;
N_mu=size(ww,2);
%% We are gonna bring to convergence the exp of the utility functions to
%% avoid problems with -\infty
U1=exp(U1); U1(:,1)=zeros(S,1);                  
U2=exp(U2); U2(:,N_mu)=zeros(S,1); 

%% The optimal consumption policies are known
for s=1:S
    for m=1:N_mu
        c(s,m,1)=( (k*(ww(m)/(1-ww(m)))/(1+k*(ww(m)/(1-ww(m))))*e(s,1))^(alp)*...
            ((ww(m)/(1-ww(m)))/(k+(ww(m)/(1-ww(m))))*e(s,2))^(1-alp));
        c(s,m,2)=( (1/(1+k*(ww(m)/(1-ww(m))))*e(s,1))^(1-alp)*...
            (k/(k+(ww(m)/(1-ww(m))))*e(s,2))^(alp));
    end
    %% At 0 and 1, one agent is dead
    c(s,N_mu,1)=e(s,1)^(alp) * e(s,2)^(1-alp);
    c(s,1,2)=e(s,1)^(1-alp) * e(s,2)^(alp);
end

%% The utility in exp is
%% U_t = c^(1-del)*exp{ del*theta *log E_t U_{t+1}^{1/theta}   }
%% Raise consumption to the appropriate power
c=c.^(1-del);

%% We are ready to start the convergence loop       
clear U1_old U2_old;
diff=1;
iter=1;
while diff>10e-12 & iter<500
    
    iter=iter+1;
    
    U1_old =U1;
    U2_old =U2;
    
    %% Future utilities
    for s=1:S
        Up1(s,:)=interp1(ww(1,:),U1(s,:),w_inv(s,:),'spline');
        Up2(s,:)=interp1(ww(1,:),U2(s,:),w_inv(s,:),'spline');
    end
    
    %%%%% Expected continuation utilities
    EUp1(1,2:N_mu)=((Up1(:,2:N_mu).^(1/th))'*p)';
    EUp2(1,1:N_mu-1)=((Up2(:,1:N_mu-1).^(1/th))'*p)';
    
    %% Update utilities    
    for s=1:S
        U1(s,2:N_mu)=c(s,2:N_mu,1).*exp(del*th*log(EUp1(1,2:N_mu)));
        U2(s,1:N_mu-1)=c(s,1:N_mu-1,2).*exp(del*th*log(EUp2(1,1:N_mu-1)));
    end
    
    %% Add back the extrema
    U1(:,1)=zeros(S,1);                  
    U2(:,N_mu)=zeros(S,1); 

    %% Convergence criterion
    diff=max(max(abs(U1-U1_old)))+max(max(abs(U2-U2_old)));
    disp(diff)
    
end

%%% Going back to the levels
U1=log(U1); U2=log(U2);
Up1=log(Up1); Up2=log(Up2);
clear EUp*
EUp1(1,2:N_mu-1)=(exp(Up1(:,2:N_mu-1)'/th)*p)';
EUp2(1,2:N_mu-1)=(exp(Up2(:,2:N_mu-1)'/th)*p)';

% A=exp(Up1(2,2:N_mu-1)/th)./EUp1(1,2:N_mu-1);
% B=exp(Up2(2,2:N_mu-1)/th)./EUp2(1,2:N_mu-1);
% plot([A'./B'],'r')
% hold on
% plot([w_inv(2,:)'./(1-w_inv(2,:)')./(ww'./(1-ww'))],'k')

%%% Save the utilities
save U1.txt U1 -ascii -double
save U2.txt U2 -ascii -double 