%% Housekeeping %%%%%
clear all; close all;
%%%%%%%%%%%%%%%%%%%%%%
load step.txt
first=38*step+1;
last =39*step;
%% Remember to check saved files at the end!!
%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load common variables
%%%%%%%%%%%%%%%%%%%%%%%%%
global options eps_der;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load U1.txt
load U2.txt
U(:,:,1)=U1; U(:,:,2)=U2; clear U1 U2;
load Q.txt 
load Q_low.txt 
load Q_top.txt 
load incr.txt 
load eps_der.txt
load p.txt
load S.txt
load e.txt 
load w_vec.txt 
load N_mu.txt 
load grid_num.txt  
load gam.txt  
load th.txt 
load del.txt  
load k.txt  
load alp.txt 
%%%%%%%%%%%%%%%%%%%%%%%%%%%
options=optimset('TolX',incr,'TolFun',incr,'Display','off');
warning off MATLAB:divideByZero
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Value function iteration: here we start
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

run der_Q %% Derivative of planner's utility

%% This is grid on the values of the utility of agent 1 in state 1
for i=first:last
    U1(1,i)=prctile(U(1,:,1),100/(grid_num+1)*i);
end

%% For each point on the previous grid we compute the utility of agents
%% 1 and 2 in every other state, as well as the optimal choice of
%% tomorrow's Pareto weights
iter2=1; %% Another little counter
for i=first:last
    
    %% Counter display
    if i/(last-first+1)*100>=iter2
        fprintf('%3.0f...',iter2);
        iter2=iter2+1;
    else end
    
    %% Given the utility of agent 1 in state 1, this computes the
    %% utility of agent 2 in state 1
    U2(1,i)=fsolve('find_U2',U(1,i,2),options,w_vec,Q,DQ,U1(1,i),1);
    
    %% Now we are ready to compute the utilities of the two agents in
    %% any other state
    for s1=2:S
        ind=max([1,min(find(U(1,:,1)>=U1(1,i)))]);
        U1(s1,i)=fsolve('foc',U(s1,ind,1),options,U1(1,i),w_vec,Q,DQ,s1,N_mu,th);
        U2(s1,i)=fsolve('find_U2',U(s1,ind,2),options,w_vec,Q,DQ,U1(s1,i),s1);
    end
    
    %% The next two lines compute the Pareto weight on agent one today
    %% for which the conitnuation utilities that we just computed are
    %% optimal
    LHS(1,i)=foc_U1_1(U1(1,i),w_vec,Q,DQ,N_mu,th,S,p,U,U1(:,i),U2(:,i));
    ww(1,i)=-LHS(1,i)/(1-LHS(1,i));
    
end

%% The next few lines are not crucial, but just in case 
%% some numerical errors in computing the derivatives 
%% generated some zeros in ww, we are gonna get rid of those
%% Normally this does not happen!
A=[ww' U1(:,:)' U2(:,:)'];
B=sortrows(A);
ind_min=min(find(B(:,1)>0));
MM=B(ind_min:end,:);
clear ww U1 U2;
ww=MM(:,1)';
U1=MM(:,2:S+1)';
U2=MM(:,S+2:2*S+1)';

%% The next loop(s) compute the state by state 
%% optimal Pareto weights tomorrow 
for i=1:size(U1,2)
    for s1=1:S
        
        DQ_val=U1(s1,i)-U2(s1,i);
        if DQ_val<=min(DQ(s1,:))
            ind=1;
            w_inv(s1,i)=w_vec(ind);
        elseif DQ_val>=max(DQ(s1,:))
            ind=size(w_vec,2);
            w_inv(s1,i)=w_vec(ind);
        else 
            ind=max(find(DQ(s1,:)<=DQ_val));
            w_inv(s1,i)=fsolve('find_w',(w_vec(ind)+w_vec(ind+1))/2,options,w_vec,DQ,DQ_val,ind,s1);
        end
        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Planner's utility    %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
for s=1:S
    for m=1:size(ww,2)
        %% We already know what the optimal consumptions are
        cc(s,m,1)=(1-del)*log( (k*(ww(m)/(1-ww(m)))/(1+k*(ww(m)/(1-ww(m))))*e(s,1))^(alp)*...
            ((ww(m)/(1-ww(m)))/(k+(ww(m)/(1-ww(m))))*e(s,2))^(1-alp));
        cc(s,m,2)=(1-del)*log( (1/(1+k*(ww(m)/(1-ww(m))))*e(s,1))^(1-alp)*...
            (k/(k+(ww(m)/(1-ww(m))))*e(s,2))^(alp));
        %% This is the planner's utility
        QQ(s,m)=ww(m)*(cc(s,m,1)+del*th*log(exp(U1(:,m)'/th)*p))+...
            (1-ww(m))*(cc(s,m,2)+del*th*log(exp(U2(:,m)'/th)*p));
    end
end


%% Save some stuff
save QQ_39.txt QQ -ascii -double
save wp_39.txt w_inv -ascii -double
save ww_39.txt ww -ascii -double
save U1_39.txt U1 -ascii -double
save U2_39.txt U2 -ascii -double




