%% Housekeeping %%%%%
clear all; close all;
%%%%%%%%%%%%%%%%%%%%%
%% Model's parameters
%%%%%%%%%%%%%%%%%%%%%
gam=10;
th=1/(1-gam);
alp=0.98; k=alp/(1-alp);
del=.95;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Vector of Pareto weights
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N_mu=10001; %10011
grid_num=10; %100
w_vec=linspace(0,1,N_mu);
% w_vec=[w_vec linspace(.101,.89,N_mu/3)];
% w_vec=[w_vec linspace(.9,1,N_mu/3)];
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Endowments
%%%%%%%%%%%%%%%%%%%%%%%%%%%
% e = [103 100; 100 103];
X = [103 100 60]; 
X = [103 100 60];
% Matrix of endowments
for s1=1:size(X,2)
    for s2=1:size(X,2)
        e((s1-1)*size(X,2)+s2,:)=[X(s1) X(s2)];
    end
end
S=size(e,1);
%% Probability of state vector
% p = [.25; .25; .25; .25];
p_r=1/500;
p_nr=(1-5*p_r)/4;
p = [p_nr p_nr p_r p_nr p_nr p_r p_r p_r p_r]';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Inital guess for Planner's utility
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% We need a good initial guess for this
%% We can either start by assuming that the Pareto weights are
%% time-invariant (init_guess_option=1), or by using some previously
%% computed Planner's utility (init_guess_option=2)
init_guess_option=1;
if init_guess_option==1
    run initialize
else
    load Q_interpolated.txt
    Q=Q_interpolated;
end
%% The first and last points are easy to compute since the Pareto weight
%% does not change over time there
Q_low=Q(:,1);
Q_top=Q(:,N_mu);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Global variables: these are some options for the various optimizations
%% in the program
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global options eps_der;
incr=1.0e-20;
eps_der=1.0e-5;
options=optimset('TolX',incr,'TolFun',incr,'Display','off');
warning off MATLAB:divideByZero
%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Value function iteration: here we start
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
diff=1;
iter=0;
while diff>.0001 & iter<1
    
    iter=iter+1; %% This is a counter on the # of iterations
    run der_Q %% Derivative of planner's utility
    
    %% This is grid on the values of the utility of agent 1 in state 1
    for i=1:grid_num
        U1(1,i)=prctile(U(1,:,1),100/(grid_num+1)*i);
    end
    
    %% For each point on the previous grid we compute the utility of agents
    %% 1 and 2 in every other state, as well as the optimal choice of
    %% tomorrow's Pareto weights
    iter2=1; %% Another little counter
    for i=1:grid_num
        
        %% Counter display
        if i/grid_num*100>=iter2
            fprintf('%3.0f...',iter2);
            iter2=iter2+1;
        else end
        
        %% Given the utility of agent 1 in state 1, this computes the
        %% utility of agent 2 in state 1
        U2(1,i)=fsolve('find_U2',U(1,i,2),options,w_vec,Q,DQ,U1(1,i),1);
        
        %% Now we are ready to compute the utilities of the two agents in
        %% any other state
        for s1=2:S
            ind=max([1,min(find(U(1,:,1)>=U1(1,i)))]);
            U1(s1,i)=fsolve('foc',U(s1,ind,1),options,U1(1,i),w_vec,Q,DQ,s1,N_mu,th);
            U2(s1,i)=fsolve('find_U2',U(s1,ind,2),options,w_vec,Q,DQ,U1(s1,i),s1);
        end
        
        %% The next two lines compute the Pareto weight on agent one today
        %% for which the conitnuation utilities that we just computed are
        %% optimal
        LHS(1,i)=foc_U1_1(U1(1,i),w_vec,Q,DQ,N_mu,th,S,p,U,U1(:,i),U2(:,i));
        ww(1,i)=-LHS(1,i)/(1-LHS(1,i));

    end
    
    %% The next few lines are not crucial, but just in case 
    %% some numerical errors in computing the derivatives 
    %% generated some zeros in ww, we are gonna get rid of those
    %% Normally this does not happen!
    A=[ww' U1(:,:)' U2(:,:)'];
    B=sortrows(A);
    ind_min=min(find(B(:,1)>0));
    MM=B(ind_min:end,:);
    clear ww U1 U2;
    ww=MM(:,1)';
    U1=MM(:,2:S+1)';
    U2=MM(:,S+2:2*S+1)';
    
    %% The next loop(s) compute the state by state 
    %% optimal Pareto weights tomorrow 
    for i=1:size(U1,2)
        for s1=1:S
            
            DQ_val=U1(s1,i)-U2(s1,i);
            if DQ_val<=min(DQ(s1,:))
                ind=1;
                w_inv(s1,i)=w_vec(ind);
            elseif DQ_val>=max(DQ(s1,:))
                ind=size(w_vec,2);
                w_inv(s1,i)=w_vec(ind);
            else 
                ind=max(find(DQ(s1,:)<=DQ_val));
                w_inv(s1,i)=fsolve('find_w',(w_vec(ind)+w_vec(ind+1))/2,options,w_vec,DQ,DQ_val,ind,s1);
            end
            
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Planner's utility    %%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    for s=1:S
        for m=1:size(ww,2)
            %% We already know what the optimal consumptions are
            cc(s,m,1)=(1-del)*log( (k*(ww(m)/(1-ww(m)))/(1+k*(ww(m)/(1-ww(m))))*e(s,1))^(alp)*...
                ((ww(m)/(1-ww(m)))/(k+(ww(m)/(1-ww(m))))*e(s,2))^(1-alp));
            cc(s,m,2)=(1-del)*log( (1/(1+k*(ww(m)/(1-ww(m))))*e(s,1))^(1-alp)*...
                (k/(k+(ww(m)/(1-ww(m))))*e(s,2))^(alp));
            %% This is the planner's utility
            QQ(s,m)=ww(m)*(cc(s,m,1)+del*th*log(exp(U1(:,m)'/th)*p))+...
                (1-ww(m))*(cc(s,m,2)+del*th*log(exp(U2(:,m)'/th)*p));
        end
    end
    
    %% Add first and last points: these are tricky to compute in the loops,
    %% because Matlab goes crazy with -Inf and such. However we already
    %% know what these are
    ww      =[0 ww 1];
    w_inv   =[zeros(S,1) w_inv ones(S,1)];
    U1      =[U(:,1,1) U1 U(:,N_mu,1)];
    U2      =[U(:,1,2) U2 U(:,N_mu,2)];
    QQ      =[Q_low QQ Q_top];
        
    %% Save some stuff
    save QQ.txt QQ -ascii
    save wp.txt w_inv -ascii
    save ww.txt ww -ascii
    save U1.txt U1 -ascii
    save U2.txt U2 -ascii
    
    %% Updating
    Q_old=Q;
    clear Q U
    for s1=1:S
        Q(s1,:)  =interp1(ww,QQ(s1,:),w_vec);
        U(s1,:,1)=interp1(ww,U1(s1,:),w_vec);
        U(s1,:,2)=interp1(ww,U2(s1,:),w_vec);
    end
    
    %% This is the convergence criterion
    diff=max(max(abs(Q-Q_old)));
    disp(' ')
    disp([iter diff])
    disp(' ')
    
    clear QQ U1 U2 ww w_inv
    
    %% Save interpolated results on the original grid
    U1_interp=U(:,:,1);
    U2_interp=U(:,:,2);
    
    save U1_interpolated.txt U1_interp -ascii
    save U2_interpolated.txt U2_interp -ascii
    save Q_interpolated.txt Q -ascii
    save w_vec.txt w_vec -ascii
    
end

run plot_results
