%% Housekeeping %%%%%
clear all; close all; clc;
%%%%%%%%%%%%%%%%%%%%%%
warning off all
%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load common variables
%%%%%%%%%%%%%%%%%%%%%%%%%
load p.txt
load S.txt
load e.txt 
load w_vec.txt 
load N_mu.txt 
load grid_num.txt  
load gam.txt  
load th.txt 
load del.txt  
load k.txt  
load alp.txt 

load U1.txt
load U2.txt
load ww.txt
load wp.txt
%%%%%%%%%%%%%%%%%%%%%%%%%

mu_grid=ww;
N_mu=size(ww,2);
mu=wp;
U(:,:,1)=U1;
U(:,:,2)=U2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Optimal consumptions %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% We already know what they are
for s=1:S
    for m=1:N_mu
        c(s,m,1)=( (k*(ww(m)/(1-ww(m)))/(1+k*(ww(m)/(1-ww(m))))*e(s,1))^(alp)*...
            ((ww(m)/(1-ww(m)))/(k+(ww(m)/(1-ww(m))))*e(s,2))^(1-alp));
        c(s,m,2)=( (1/(1+k*(ww(m)/(1-ww(m))))*e(s,1))^(1-alp)*...
            (k/(k+(ww(m)/(1-ww(m))))*e(s,2))^(alp));
    end
    %% At 0 and 1, one agent is dead
    c(s,N_mu,1)=e(s,1)^(alp) * e(s,2)^(1-alp);
    c(s,1,2)=e(s,1)^(1-alp) * e(s,2)^(alp);
end

%% Future consumption in logs
for i=1:2
    for s=1:S
        cp(s,:,i)=interp1(mu_grid(1,:),c(s,:,i),mu(s,:));
    end
end


c=(1-del)*log(c);
cp=log(cp);



U=exp(U);
%%% I need to create the continuation utilities with interp1.....
%% Future consumption in logs
for i=1:2
    for s=1:S
        Up(s,:,i)=interp1(ww(1,:),U(s,:,i),mu(s,:),'spline');
    end
end
U=log(U);
Up=log(Up);





%%%%% Expected continuation utilities
    EU(1,2:N_mu-1,1)=exp(Up(:,2:N_mu-1,1)/th)'*p;
    EU(1,2:N_mu-1,2)=exp(Up(:,2:N_mu-1,2)/th)'*p;
    


%% SDF's at t+1
for i=1:2
    for s=1:S
        for sp=1:S
            for m=2:N_mu-1
                %% Log sdf
                sdfp(s,m,sp,i)=log(del)-cp(sp,m,i)+c(s,m,i)/(1-del)+...
                    1/th*Up(sp,m,i)-log(EU(1,m,i));
                %% Level SDF
                SDFp(s,m,sp,i)=exp(log(del)-cp(sp,m,i)+c(s,m,i)/(1-del)+...
                    1/th*Up(sp,m,i)-log(EU(1,m,i)));
            end
        end
    end
end

%% Consumption growth
for i=1:2
    for s=1:S
        for sp=1:S
            for m=2:N_mu-1
                dcp(s,m,sp,i)=cp(sp,m,i)-c(s,m,i)/(1-del);
            end
        end
    end
end


%%% Expected consumption growth variance and mean
for s=1:S
    for m=2:N_mu-1
        Edc(s,m,1)=(reshape(dcp(s,m,:,1),1,S)*p);      
        Edc(s,m,2)=(reshape(dcp(s,m,:,2),1,S)*p);
        Vdc(s,m,1)=((reshape(dcp(s,m,:,1),1,S)-Edc(s,m,1)).^2)*p;      
        Vdc(s,m,2)=((reshape(dcp(s,m,:,2),1,S)-Edc(s,m,2)).^2)*p; 
        Vdc1_dc2(s,m)=((reshape(dcp(s,m,:,1),1,S)-reshape(dcp(s,m,:,2),1,S)-Edc(s,m,1)+Edc(s,m,2)).^2)*p;
        corr_dc(s,m,1)=((reshape(dcp(s,m,:,1),1,S)-Edc(s,m,1)))...
            .*((reshape(dcp(s,m,:,2),1,S)-Edc(s,m,2)))*p./(sqrt(Vdc(s,m,1).*Vdc(s,m,2)));
        for i=1:2
            E_Up(m,i)=Up(:,m,i)'*p;
            V_Up(m,i)=(Up(:,m,i)'-E_Up(m,i)).^2*p;
                        corr_Up_dc(s,m,i)=(Up(:,m,i)'-E_Up(m,i)).*...
                ((reshape(dcp(s,m,:,i),1,S)-Edc(s,m,i)))*p./sqrt(Vdc(s,m,i).*V_Up(m,i));
        end
        V_Up1_Up2(m)=(Up(:,m,1)'-E_Up(m,1)-Up(:,m,2)'+E_Up(m,2)).^2*p;
        corr_Up12_dc12(s,m)=(Up(:,m,1)'-E_Up(m,1)-Up(:,m,2)'+E_Up(m,2)).*...
            (reshape(dcp(s,m,:,1),1,S)-reshape(dcp(s,m,:,2),1,S)-Edc(s,m,1)+Edc(s,m,2))*p./sqrt(Vdc1_dc2(s,m).*V_Up1_Up2(m));            
        
    end
end



%%% Risk-free rates and expected depreciation rates
for s=1:S
    for m=2:N_mu-1
        rf(s,m,1)=-log(reshape(SDFp(s,m,:,1),1,S)*p);      
        rf(s,m,2)=-log(reshape(SDFp(s,m,:,2),1,S)*p);
        Edfx(s,m)=(reshape(sdfp(s,m,:,2),1,S)-reshape(sdfp(s,m,:,1),1,S))*p;
        Vdfx(s,m)=(reshape(sdfp(s,m,:,2),1,S)-reshape(sdfp(s,m,:,1),1,S)-Edfx(s,m)).^2*p;
        corr_dfx_dc1dc2(s,m)=(reshape(sdfp(s,m,:,2),1,S)-reshape(sdfp(s,m,:,1),1,S)-Edfx(s,m)).*...
            ((reshape(dcp(s,m,:,1),1,S)-reshape(dcp(s,m,:,2),1,S)-Edc(s,m,1)+Edc(s,m,2)))*p./sqrt(Vdc1_dc2(s,m).*Vdfx(s,m));
    
    end
end

%% Summary statistics
load g1.txt

%% UIP regressions
for s=1:S
mean_dfx(s,1)=Edfx(s,:)*g1(1:N_mu-1,1)/sum(g1);
mean_drf(s,1)=(rf(s,:,1)-rf(s,:,2))*g1(1:N_mu-1,1)/sum(g1);
cov(s,1)=(((Edfx(s,:)-mean_dfx(s,1)).*(rf(s,:,1)-rf(s,:,2)-mean_drf(s,1))))*g1(1:N_mu-1,1)/sum(g1);
var(s,1)=(((rf(s,:,1)-rf(s,:,2)-mean_drf(s,1))).*(rf(s,:,1)-rf(s,:,2)-mean_drf(s,1)))*g1(1:N_mu-1,1)/sum(g1);
bet(s,1)=cov(s,1)/var(s,1);
end

%% Risk free rates
for s=1:S
mean_rf(s,:)=rf(s,:,1).*g1(1:N_mu-1,1)'/sum(g1)*p(s);
mean_rf2(s,:)=(rf(s,:,1).^2).*g1(1:N_mu-1,1)'/sum(g1)*p(s);
end
mean_rfr=sum(sum(mean_rf));
std_rf=sqrt( sum(sum(mean_rf2)) - mean_rfr.^2   );

%% Sdfs
%% conditional moments
for s=1:S
    for m=2:N_mu-1
        for i=1:2
            Ec_sdf(s,m,i)=(reshape(SDFp(s,m,:,i),1,S))*p;
            Ec_sdf2(s,m,i)=((reshape(SDFp(s,m,:,i),1,S)).^2)*p;
            Vc_sdf(s,m,i)=Ec_sdf2(s,m,i)-Ec_sdf(s,m,i)^2;
        end
        cov_sdf(s,m)=(reshape(SDFp(s,m,:,1),1,S)-Ec_sdf(s,m,1)).*...
            (reshape(SDFp(s,m,:,2),1,S)-Ec_sdf(s,m,2))*p;
        corr_sdf(s,m)=cov_sdf(s,m)/sqrt(Vc_sdf(s,m,1)*Vc_sdf(s,m,2));
    end
end
%%% unconditional moments
for s=1:S
mean_sdf(s,:)=Ec_sdf(s,:,1).*g1(1:N_mu-1,1)'/sum(g1)*p(s);
mean_sdf2(s,:)=(Ec_sdf2(s,:,1)).*g1(1:N_mu-1,1)'/sum(g1)*p(s);
end
mean_sdfs=sum(sum(mean_sdf));
std_sdf=sqrt( sum(sum(mean_sdf2)) - mean_sdfs.^2   );
%%% Market price of risk
HJ=std_sdf./mean_sdfs;


%% Exchange rates
for s=1:S
mean_Edfx(s,:)=Edfx(s,:,1).*g1(1:N_mu-1,1)'/sum(g1)*p(s);
mean_Edfx2(s,:)=(Edfx(s,:,1).^2).*g1(1:N_mu-1,1)'/sum(g1)*p(s);
end
mean_Edfxx=sum(sum(mean_Edfx));
std_Edfx=sqrt( sum(sum(mean_Edfx2)) - mean_Edfxx.^2   );

%% Net export-output ratio (home country)
SS(1,2:N_mu-1)  = ww(1,2:N_mu-1)./(1-ww(1,2:N_mu-1));
NX(1,2:N_mu-1) = (1-SS(1,2:N_mu-1))./(1+k*SS(1,2:N_mu-1));
mean_NX=NX*g1(1:N_mu-1,1)/sum(g1);
std_NX=sqrt(((NX-mean_NX).^2)*g1(1:N_mu-1,1)/sum(g1));

%% Some plots
first=100; last=N_mu-100;
%% Correlation of sdfs and dc across countries
figure(1)
for i=1:S
subplot(3,3,i), plotyy(ww(1,first:last)',[(corr_dc(1,first:last,1)') ],ww(1,first:last)',(corr_sdf(1,first:last,1)'));
end
[ax,h3]=suplabel('Correlations of sdfs and dc'  ,'t');
set(h3,'FontSize',14)
saveas(gcf,'figure01.eps', 'psc2')
%% Conditional volatility of sdf
figure(2)
for i=1:S
subplot(3,3,i), plot(ww(1,first:last)',100*sqrt(Vc_sdf(i,first:last,1)'),'.')
end
[ax,h3]=suplabel('Volatility of home sdf (in levels)'  ,'t');
set(h3,'FontSize',14)
saveas(gcf,'figure02.eps', 'psc2')
%% Conditional volatility of of consumption growth
figure(3)
for i=1:S
subplot(3,3,i), plot(ww(1,first:last)',100*sqrt(Vdc(i,first:last,1)'),'.')
end
[ax,h3]=suplabel('Volatility of home consumption growth (in logs)'  ,'t');
set(h3,'FontSize',14)
saveas(gcf,'figure03.eps', 'psc2')
%% Conditional volatility of exchange rate growth
figure(4)
for i=1:S
subplot(3,3,i), plot(ww(1,first:last)',100*sqrt(Vdfx(i,first:last,1)'),'.')
end
[ax,h3]=suplabel('Volatility of exchange rate growth (in logs)'  ,'t');
set(h3,'FontSize',14)
saveas(gcf,'figure04.eps', 'psc2')
%% Conditional expectation of exchange rate growth
figure(5)
for i=1:S
subplot(3,3,i), plot(ww(1,first:last)',100*(Edfx(i,first:last,1)'),'.')
end
[ax,h3]=suplabel('Expected exchange rate growth (in logs)'  ,'t');
set(h3,'FontSize',14)
saveas(gcf,'figure05.eps', 'psc2')
%% Risk free rates
figure(6)
for i=1:S
subplot(3,3,i), plot(ww(1,first:last)',[100*(rf(i,first:last,1)') 100*(rf(i,first:last,2)')],'.')
end
[ax,h3]=suplabel('Risk-free rates (in logs)'  ,'t');
set(h3,'FontSize',14)
saveas(gcf,'figure06.eps', 'psc2')
%% UIP regressions
figure(7)
for i=1:S
subplot(3,3,i), plot([100*(rf(i,first:last,1)')-100*(rf(i,first:last,2)')],100*(Edfx(i,first:last,1)'),'.')
end
[ax,h3]=suplabel('UIP regressions'  ,'t');
set(h3,'FontSize',14)
saveas(gcf,'figure07.eps', 'psc2')
%% NX/output ratio
figure(8)
plot(ww(1,first:last)',100*(NX(1,first:last)') ,'.')
title('NX/output ratio')
saveas(gcf,'figure08.eps', 'psc2')
%% NX and FX volatility
figure(9)
for i=1:S
subplot(3,3,i), plot(100*sqrt(Vdfx(i,first:5001,1)'),100*(NX(1,first:5001)'),'.')
end
[ax,h3]=suplabel('Exchange rate volatility and NX'  ,'t');
set(h3,'FontSize',14)
saveas(gcf,'figure09.eps', 'psc2')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Display results %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('=============================')
disp('RESULTS')
disp('=============================')
disp('=============================')
disp('Consumption growth')
disp('-----------------------------')
fprintf('Mean: %6.3f\n',(Edc(:,:,1)*g1(1:N_mu-1)/sum(g1))'*p*100);
fprintf('Std: %6.3f\n',sqrt((Vdc(:,:,1)*g1(1:N_mu-1)/sum(g1))'*p)*100);
% disp('State by state mean:')
% disp((Edc(:,:,1)*g1(1:N_mu-1)/sum(g1))'*100)
% disp('State by state std:')
% disp(sqrt((Vdc(:,:,1)*g1(1:N_mu-1)/sum(g1))')*100)
disp('=============================')
disp('Stochastic Discount Factor')
disp('-----------------------------')
fprintf('Mean: %6.3f\n',(Ec_sdf(:,:,1)*g1(1:N_mu-1)/sum(g1))'*p*100);
fprintf('Std: %6.3f\n',sqrt((Vc_sdf(:,:,1)*g1(1:N_mu-1)/sum(g1))'*p)*100);
fprintf('HJ bound: %6.3f\n',sqrt((Vc_sdf(:,:,1)*g1(1:N_mu-1)/sum(g1))'*p)/((Ec_sdf(:,:,1)*g1(1:N_mu-1)/sum(g1))'*p)*100);
disp('=============================')
disp('Risk-free rates')
disp('-----------------------------')
fprintf('Mean: %6.3f\n',mean_rfr*100);
fprintf('Std: %6.3f\n',std_rf*100);
disp('=============================')
disp('Exchange rates')
disp('-----------------------------')
fprintf('Mean: %6.3f\n',(Edfx(:,:)*g1(1:N_mu-1)/sum(g1))'*p*100);
fprintf('Std: %6.3f\n',sqrt((Vdfx(:,:)*g1(1:N_mu-1)/sum(g1))'*p)*100);
disp('=============================')
disp('UIP regressions slope')
disp('-----------------------------')
fprintf('Mean: %6.3f\n',bet'*p);
disp('=============================')
disp('International correlations')
disp('-----------------------------')
fprintf('Cons. growth: %6.3f\n',(corr_dc(:,:,1)*g1(1:N_mu-1)/sum(g1))'*p);
fprintf('SDFs : %6.3f\n',sqrt((corr_sdf(:,:,1)*g1(1:N_mu-1)/sum(g1))'*p));
disp('=============================')
disp('NX/output')
disp('-----------------------------')
fprintf('Mean: %6.3f\n',(NX(:,:)*g1(1:N_mu-1)/sum(g1))*100);
fprintf('Std: %6.3f\n', sqrt( (NX(:,:).^2*g1(1:N_mu-1)/sum(g1)) - (NX(:,:)*g1(1:N_mu-1)/sum(g1))^2 )*100  );
disp('=============================')

diary summary_statistics